@@ -31,24 +31,27 @@ Luke Arno can be found at http://lukearno.com/
 
 """
 
-import types
 import re
+from xml.sax.saxutils import escape
 
-# this bit adapted from O'Reilly Python Cookbook
-edict = {'&':'&amp;', '<':'&lt;','>':'&gt;'}
-rx = re.compile('|'.join(map(re.escape, edict)))
-xlat = lambda match: edict[match.group(0)]
 
-def escape(astr):
-    """Escape entities for XML."""
-    return rx.sub(xlat, astr)
+def isiterable(it):
+    """return True if 'it' is iterable else return False."""
+    try:
+        iter(it)
+    except:
+        return False
+    else:
+        return True
+
 
 def make_attributes(dct):
     """Turn a dict into string of XML attributes."""
-    return "".join((' %s="%s"' % (x, escape(str(y))) 
-                    for x, y in dct.iteritems()))
+    return u"".join((' %s="%s"' % (x, escape(unicode(y))) 
+                     for x, y in dct.iteritems()))
 
-def pyfo(node, prolog=False, pretty=False, indent_size=2):
+
+def pyfo(node, prolog=False, pretty=False, indent_size=2, encoding='utf-8'):
     """Generate XML using native python data structures.
    
     node structure like (name, contents) or (name, contents, attribs)
@@ -69,19 +72,19 @@ def pyfo(node, prolog=False, pretty=False, indent_size=2):
             if it is a list, pyfo is called on all its members 
             and the results are concatenated to become the contents.
 
-            otherwise it is run through 'str' and 'escape'.
+            otherwise it is run through 'unicode' and 'escape'.
     
         optional third item: 
             should be a dictionary used as xml attributes
     
     for a string:
         
-        just return it.
+        just return it as unicode.
     """
     if callable(node):
         node = node()
     if not node:
-        return ""
+        return u""
     if pretty and pretty >= 0:
         if pretty is True: pretty = 1
         indent = '\n' + (" " * indent_size * pretty)
@@ -90,7 +93,7 @@ def pyfo(node, prolog=False, pretty=False, indent_size=2):
     else:
         unindent = indent = ""
     if isinstance(node, basestring):
-        return node
+        return unicode(node)
     elif len(node) == 2:
         name, contents = node
         dct = {}
@@ -99,32 +102,33 @@ def pyfo(node, prolog=False, pretty=False, indent_size=2):
     leaf = False
     if callable(contents):
         contents = contents()
-        leaf = True
     if isinstance(contents, dict):
         contents = contents.items()
-    # How can this catch more iterables?
-    if type(contents) in [types.ListType, types.GeneratorType]:
+    if isinstance(contents, tuple):
+        contents = pyfo(contents, pretty=pretty, indent_size=indent_size)
+    elif not isinstance(contents, basestring) and isiterable(contents):
         cgen = (pyfo(c, pretty=pretty, indent_size=indent_size) 
                 for c in contents)
         contents = indent.join((c for c in cgen if c))
-    elif type(contents) is types.TupleType:
-        contents = pyfo(contents, pretty=pretty, indent_size=indent_size)
     elif contents not in [None, ""]:
-        contents = escape(str(contents))
+        contents = escape(unicode(contents))
         leaf = True
     if leaf:
         indent = unindent = ""
-    prolog = prolog and '<?xml version="1.0"?>\n' or ''
-    if contents:
-        return '%s<%s%s>%s%s%s</%s>' % (prolog, 
-                                        name, 
-                                        make_attributes(dct), 
-                                        indent,
-                                        contents, 
-                                        unindent,
-                                        name)
+    if prolog:
+        prolog = u'<?xml version="1.0" encoding="%s"?>\n' % encoding
     else:
-        return '%s<%s%s/>' % (prolog, 
-                              name, 
-                              make_attributes(dct))
+        prolog = u''
+    if contents:
+        return u'%s<%s%s>%s%s%s</%s>' % (prolog, 
+                                         name, 
+                                         make_attributes(dct), 
+                                         indent,
+                                         contents, 
+                                         unindent,
+                                         name)
+    else:
+        return u'%s<%s%s/>' % (prolog, 
+                               name, 
+                               make_attributes(dct))
 
